let instanceCount = 0;


export default {

    name: 'field-layout',

    _meta: {
        label: 'Layout',
        info: 'Component based layouts',
        icon: 'layout:assets/icons/layout.svg',
        settings: [
            {
                name: 'allowedComponents',
                type: 'select',
                label: 'Components',
                info: 'List of components to pick from.',
                opts: {
                    multiple: true,
                    src: {
                        route: `/layout/components?list=1&v=${App.version}`,
                        map: {
                            value: 'name',
                            label: 'label',
                        }
                    }
                }
            },
        ],
    },

    data() {
        return {
            uid: `field-layout-${++instanceCount}`,
            val: this.modelValue || [],
            ready: false,
            showPreview: this.preview,
            actionComponent: null,
            isDragging: false,

            globalComponents: null,
        }
    },

    mounted() {

        (this.val || []).forEach((component, idx) => {

            if (!component.id) {
                this.val[idx] = Object.assign({ id: App.utils.nanoid() }, component);
            }
        });

        App.utils.getlayoutComponents().then(components => {

            this.globalComponents = components;
            this.ready = true;
        });

        if (this.level === 0) {

            const rootEl = this.$el.parentNode;

            KISS.events.on(rootEl, 'mouseover', '.layout-component-container', function(e) {

                e.stopPropagation();

                this.classList.add('isHovering');

                const parent = this.parentNode.closest('.layout-component-container.isHovering');

                if (parent) parent.classList.remove('isHovering');
            });

            KISS.events.on(rootEl, 'mouseout', '.layout-component-container', function(e) {
                this.classList.remove('isHovering');
            });
        }
    },

    props: {
        modelValue: {
            type: Array,
            default: []
        },
        group: {
            type: String,
            default: null
        },
        level: {
            type: Number,
            default: 0
        },
        preview: {
            type: Boolean,
            default: true
        },
        allowedComponents: {
            type: Array,
            default: []
        },
    },

    components: {
        gridComponent: Vue.defineAsyncComponent(() =>
            App.utils.import('layout:assets/vue-components/layout/grid.js')
        ),
        rowComponent: Vue.defineAsyncComponent(() =>
            App.utils.import('layout:assets/vue-components/layout/row.js')
        ),
        componentPreview: Vue.defineAsyncComponent(() =>
            App.utils.import('layout:assets/vue-components/layout/component-preview.js')
        )
    },

    watch: {
        val: {
            handler() { this.update() },
            deep: true
        },
        modelValue: {
            handler(val) {
                this.val = this.modelValue || [];
                this.update();
            },
            deep: true
        },
        preview(val) {
            this.showPreview = val;
        }
    },

    methods: {

        addComponent(previous) {

            let components = {}, restrict = Array.isArray(this.allowedComponents) && this.allowedComponents.length ? this.allowedComponents : Object.keys(this.globalComponents);

            Object.keys(this.globalComponents).forEach(name => {

                if (restrict.includes(name)) {
                    components[name] = this.globalComponents[name];
                }
            });

            VueView.ui.offcanvas('layout:assets/vue-components/layout/component-picker.js', {components}, {

                select: component => {

                    let meta = this.globalComponents[component], data = {};

                    (meta.fields || []).forEach(field => {
                        data[field.name] = field.default || null;
                    });

                    let item = {
                        id: App.utils.nanoid(),
                        component,
                        label: meta.label || component,
                        children: meta.children ? [] : null,
                        data
                    };

                    VueView.ui.modal('layout:assets/vue-components/layout/component-edit.js', {component: item, mode: 'add', globalComponents: this.globalComponents}, {

                        save: component => {

                            Object.assign(item, component);

                            if (previous) {
                                this.val.splice(this.val.indexOf(previous) + 1, 0, item);
                            } else {
                                this.val.push(item);
                            }
                        }
                    }, {size: meta.opts?.['dialog.size'] || 'large'})
                },

                selectFromClipboard: component => {

                    component.id = App.utils.nanoid();

                    if (previous) {
                        this.val.splice(this.val.indexOf(previous) + 1, 0, component);
                    } else {
                        this.val.push(component);
                    }
                }

            }, {flip: true, size: 'medium'})

        },

        edit(item) {

            let meta = this.globalComponents[item.component];

            if (!meta) {
                App.ui.notify(`Component definition for "${item.component}" not found`, 'error');
                return;
            }

            VueView.ui.modal('layout:assets/vue-components/layout/component-edit.js', {component: item, mode: 'edit', globalComponents: this.globalComponents}, {

                save: component => {
                    Object.assign(item, component);
                }

            }, {size: meta.opts?.['dialog.size'] || 'large'})
        },

        remove(item) {
            this.val.splice(this.val.indexOf(item), 1);
            this.update();
        },

        duplicate(item) {

            let cItem = JSON.parse(JSON.stringify(item));

            cItem.id = App.utils.nanoid();

            this.val.splice(this.val.indexOf(item) + 1, 0, cItem);
            this.update();

            this.refreshComponentPreview();
        },

        paste(item) {

            const cItem = this.clipboardComponent();

            if (!cItem || !this.globalComponents[cItem.component]) {
                return false;
            }

            cItem.id = App.utils.nanoid();

            this.val.splice(this.val.indexOf(item) + 1, 0, cItem);
            this.update();

            this.refreshComponentPreview();
        },

        clipboardComponent() {
            return JSON.parse(App.storage.get('layout.field.clipboard.component', 'null', true));
        },

        copy(item) {

            let cItem = JSON.parse(JSON.stringify(item));

            App.storage.set('layout.field.clipboard.component', JSON.stringify(cItem));
            App.ui.notify('Component copied!');

            App.trigger('layout.field.clipboard.change', {component: cItem});
        },

        change(evt) {
            this.update();
        },

        update() {
            this.$emit('update:modelValue', this.val)
        },

        hasPreview(component) {
            if (
                this.globalComponents[component.component] &&
                (
                    // template preview
                    this.globalComponents[component.component].preview ||
                    //component preview Url
                    (
                        this.globalComponents[component.component].opts &&
                        this.globalComponents[component.component].opts.previewComponent
                    )
                )
            ) {
                return true;
            }

            return false;
        },

        // hack for fixing preview after duplicate/paste component
        refreshComponentPreview() {

            this.showPreview = false;

            setTimeout(() => {
                this.showPreview = true;
            }, 10);
        },

        dragStart() {

            this.isDragging = true;
            this.showPreviewAfterDragging = false;

            if (!this.showPreview) {
                return;
            }

            this.showPreviewAfterDragging = true;
            this.$emit('hide-preview')
            this.showPreview = false;

            document.documentElement.classList.add('app-sortable-active');
        },

        dragEnd(e) {

            this.isDragging = false;

            if (this.showPreviewAfterDragging) {
                this.$emit('show-preview');
                this.showPreview = true;
            }

            const componentId = e.clone && e.clone.getAttribute('data-component-id');

            if (componentId) {

                setTimeout(() => {

                    const component = document.querySelector(`[data-component-id="${componentId}"]`);

                    if (component && !KISS.utils.isInViewport(component, true)) {
                        component.scrollIntoView({behavior: 'smooth', block: 'start'});
                    }
                }, 100);
            }

            document.documentElement.classList.remove('app-sortable-active');
        }
    },

    template: /*html*/`
        <div field="layout" v-if="ready">
            <div :class="{'kiss-position-relative': (Array.isArray(val) && !val.length)}" :style="{minHeight:(Array.isArray(val) && !val.length) ? '100px':''}">
                <vue-draggable
                    :list="val"
                    :group="group || uid"
                    @change="change"
                    @start="dragStart"
                    @end="dragEnd"
                    handle=".lm-handle"
                    class="field-layout-dragarea"
                    :class="{'isDragging':isDragging}"
                    :swapThreshold="0.65"
                    :animation="150"
                    :fallbackOnBody="true"
                >
                    <template #item="{ element, index }">
                        <div class="layout-component-container kiss-position-relative kiss-margin-small" :data-component-id="element.id || ''">
                            <kiss-card
                                class="layout-component kiss-visible-toggle"
                                theme="bordered contrast"
                                hover="contrast shadow"
                            >
                                <div class="layout-component-topbar kiss-padding-small kiss-flex kiss-flex-middle" gap="small">
                                    <a class="lm-handle kiss-margin-small-right kiss-color-muted"><icon>drag_handle</icon></a>
                                    <div class="kiss-flex-1 kiss-size-xsmall kiss-text-bold kiss-text-truncate">
                                        <a class="kiss-link-muted kiss-flex kiss-flex-middle" :class="{'kiss-color-muted': element.hidden}" @click="edit(element)">
                                            <icon class="kiss-margin-xsmall-right" v-if="element.hidden">visibility_off</icon>
                                            <span>{{ element.label }}</span>
                                        </a>
                                    </div>
                                    <span class="kiss-color-muted kiss-text-caption kiss-size-xsmall kiss-invisible-hover">{{ element.component }}</span>

                                    <div class="kiss-button-group kiss-invisible-hover">
                                        <a class="kiss-button kiss-button-small" :title="t('Move up')" v-if="val.indexOf(element) !== 0" @click="val.splice(index - 1, 0, val.splice(index, 1)[0])"><icon>arrow_upward</icon></a>
                                        <a class="kiss-button kiss-button-small" :title="t('Move down')" v-if="val.indexOf(element) !== val.length - 1" @click="val.splice(index, 2, val[index + 1], val[index])"><icon>arrow_downward</icon></a>
                                    </div>

                                    <a class="kiss-invisible-hover kiss-color-muted" @click="duplicate(element)" :title="t('Duplicate')"><icon>control_point_duplicate</icon></a>
                                    <a class="kiss-invisible-hover kiss-color-muted" @click="remove(element)" :title="t('Delete')"><icon>delete</icon></a>
                                    <a class="kiss-invisible-hover" @click="actionComponent = element"><icon>more_horiz</icon></a>
                                </div>
                                <div class="kiss-padding-small field-layout-children" v-if="element.children">
                                    <field-layout v-model="element.children" :group="group || uid" :allowedComponents="allowedComponents" :level="++level" :preview="showPreview" @hide-preview="dragStart" @show-preview="showPreview = true"></field-layout>
                                </div>
                                <grid-component class="field-layout-component-grid kiss-margin-small kiss-padding-small" v-model="element" :group="group || uid" :allowedComponents="allowedComponents" :level="++level" :preview="showPreview" v-if="element.component == 'grid'"></grid-component>
                                <row-component class="field-layout-component-row kiss-margin-small kiss-padding-small" v-model="element" :group="group || uid" :allowedComponents="allowedComponents" :level="++level" :preview="showPreview" v-if="element.component == 'row'"></row-component>
                                <div class="kiss-bgcolor-contrast" v-if="!element.hidden && showPreview && !element.children && hasPreview(element)">
                                    <component class="kiss-margin-xsmall-top kiss-size-small" :is="'component-preview'" :component="element" :globalComponents="globalComponents"></component>
                                </div>
                            </kiss-card>
                            <a class="field-layout-btn-add-after animated fadeIn faster" :title="t('Add component')" @click="addComponent(element)"></a>

                        </div>
                    </template>
                </vue-draggable>

                <div class="field-layout-dragarea-empty kiss-cover kiss-flex kiss-flex-center kiss-flex-middle" v-if="Array.isArray(val) && !val.length">
                    <div class="kiss-color-muted kiss-align-center">
                        <kiss-svg :src="App.base('layout:assets/icons/layout.svg')" width="50" height="50"></kiss-svg>
                        <div><icon size="larger">add_box</icon></div>
                    </div>
                    <a class="kiss-cover" @click="addComponent()"></a>
                </div>
            </div>

            <div class="kiss-button-group kiss-margin-small" v-if="!level">
                <button type="button" class="kiss-button kiss-button-small" @click="addComponent()">{{ t('Add component') }}</button>
                <button type="button" class="kiss-button kiss-button-small" @click="showPreview = !showPreview">{{ showPreview ? t('Hide preview') : t('Show preview') }}</button>
            </div>

        </div>
        <app-loader v-else size="small"></app-loader>

        <teleport to="body">
            <kiss-popout :open="actionComponent && 'true'" @popoutclose="actionComponent = null">
                <kiss-content>
                    <kiss-navlist v-if="actionComponent">
                        <ul>
                            <li class="kiss-nav-header">{{ t('Component actions') }}</li>
                            <li>
                                <a class="kiss-flex kiss-flex-middle" @click="edit(actionComponent)">
                                    <icon class="kiss-margin-small-right">create</icon>
                                    {{ t('Edit') }}
                                </a>
                            </li>
                            <li>
                                <a class="kiss-flex kiss-flex-middle" @click="duplicate(actionComponent)">
                                    <icon class="kiss-margin-small-right">control_point_duplicate</icon>
                                    {{ t('Duplicate') }}
                                </a>
                            </li>
                            <li class="kiss-nav-divider"></li>
                            <li>
                                <a class="kiss-flex kiss-flex-middle" @click="copy(actionComponent)">
                                    <icon class="kiss-margin-small-right">content_copy</icon>
                                    {{ t('Copy') }}
                                </a>
                            </li>
                            <li>
                                <a class="kiss-flex kiss-flex-middle" :class="{'kiss-color-muted kiss-disabled': !clipboardComponent()}" @click="paste(actionComponent)">
                                    <icon class="kiss-margin-small-right">content_paste</icon>
                                    {{ t('Paste') }}
                                </a>
                            </li>
                            <li class="kiss-nav-divider" v-if="val.length > 1"></li>

                            <li v-if="val.indexOf(actionComponent) !== 0">
                                <a class="kiss-flex kiss-flex-middle" @click="val.unshift(val.splice(val.indexOf(actionComponent), 1)[0])">
                                    <icon class="kiss-margin-small-right">arrow_upward</icon>
                                    {{ t('Move first') }}
                                </a>
                            </li>
                            <li v-if="val.indexOf(actionComponent) !== val.length - 1">
                                <a class="kiss-flex kiss-flex-middle" @click="val.push(val.splice(val.indexOf(actionComponent), 1)[0])">
                                    <icon class="kiss-margin-small-right">arrow_downward</icon>
                                    {{ t('Move last') }}
                                </a>
                            </li>
                            <li class="kiss-nav-divider"></li>
                            <li>
                                <a class="kiss-color-danger kiss-flex kiss-flex-middle" @click="remove(actionComponent)">
                                    <icon class="kiss-margin-small-right">delete</icon>
                                    {{ t('Delete') }}
                                </a>
                            </li>
                        </ul>
                    </kiss-navlist>
                </kiss-content>
            </kiss-popout>
        </teleport>
    `
}
