import {getGroupColor} from './utils.js';


let pickComponent = {

    data() {
        return {
            group: null,
            filter: '',
            selected: null,
            getGroupColor
        }
    },

    props: {
        components: {
            type: Object
        }
    },

    computed: {

        clipboardComponent() {
            return JSON.parse(App.storage.get('layout.field.clipboard.component', 'null', true));
        },

        filtered() {

            let components = {}, component = null;

            Object.keys(this.components || {}).sort((x, y) => {

                // sort components
                x = (this.components[x].label || x).toLocaleLowerCase();
                y = (this.components[y].label || y).toLocaleLowerCase();

                if (x < y) return -1;
                if (x > y) return 1;

                return 0;

            }).forEach(name => {

                if (this.filter && !`${name} ${this.components[name].label || ''}`.toLocaleLowerCase().includes(this.filter.toLocaleLowerCase())) {
                    return;
                }

                if (this.group && this.components[name].group != this.group ) return;

                components[name] = this.components[name]
            });

            return components;
        },

        groups() {
            let groups = [];

            Object.keys(this.components || {}).forEach(name => {
                if (!this.components[name].group || groups.indexOf(this.components[name].group) > -1) return;
                groups.push(this.components[name].group);
            });

            return groups.sort();
        },

        selectedComponent() {
            return this.selected !== null ? Object.keys(this.filtered)[this.selected] : null;
        }
    },

    methods: {
        select(component) {
            this.$call('select', component);
            this.$close();
        },

        selectFromClipboard() {
            this.$call('selectFromClipboard', this.clipboardComponent);
            this.$close();
        },

        previewUrl(component) {

            const image = this.components[component].image || null;

            if (!image || !image.file) {
                return null;
            }

            return `uploads://buckets/${image.bucket}/${image.file.path}`;
        },

        keydown(event) {

            switch (event.keyCode) {

                // close on ESC
                case 27:
                    this.selected = null;
                    break;
                // enter
                case 13:

                    if (this.selected !== null) {
                        event.preventDefault();
                        this.select(this.selectedComponent);
                        return;
                    }

                    if (Object.keys(this.filtered).length === 1) {
                        event.preventDefault();
                        this.select(Object.keys(this.filtered)[0]);
                        return;
                    }

                    break;

                // up | down
                case 38:
                case 40:

                    const components = Object.keys(this.filtered);

                    if (!components.length) {
                        return;
                    }

                    event.preventDefault();

                    if (this.selected === null) {
                        this.selected = event.keyCode == 38 ? components.length - 1 : 0;
                    } else {

                        if (event.keyCode == 38) {
                            this.selected = components[this.selected - 1] ? this.selected - 1 : components.length - 1 ;
                        } else {
                            this.selected = components[this.selected + 1] ? this.selected + 1 : 0 ;
                        }
                    }

                    const ele = document.getElementById(`layout-component-picker-${this.selectedComponent}`);

                    if (ele) {
                        ele.scrollIntoView({behavior: 'smooth', block: 'start'});
                    }
                    break;
            }
        },
    },

    template: /*html*/`
        <div class="app-offcanvas-container">

            <div class="kiss-padding kiss-size-4 kiss-text-bold kiss-flex kiss-flex-middle">
                <div class="kiss-margin-small-right">
                    <kiss-svg :src="$baseUrl('layout:assets/icons/component.svg')" width="40" height="40"></kiss-svg>
                </div>
                {{ t('Pick a component') }}
            </div>

            <div class="kiss-padding">
                <input type="text" class="kiss-input" :placeholder="t('Filter components...')" @keydown="keydown" @input="selected=null" v-model="filter" autofocus>

                <div class="kiss-flex kiss-margin-small-top kiss-width-1-2@m kiss-overlay-input" gap>
                    <div>
                        <div>
                            <span class="kiss-text-caption kiss-color-muted">{{ t('Group') }}</span>
                            <div class="kiss-text-capitalize kiss-size-xsmall" :class="{'kiss-color-muted': !group, 'kiss-text-bold': group}">{{ group || t('All components') }}</div>
                        </div>
                        <select v-model="group">
                            <option :value="null">{{ t('All components') }}</option>
                            <option :value="name" v-for="name in groups">{{ name }}</option>
                        </select>
                    </div>
                </div>
            </div>

            <div class="app-offcanvas-content kiss-padding">

                <kiss-card :id="'layout-component-picker-'+component" class="kiss-padding-small kiss-position-relative kiss-margin-small" theme="bordered" hover="contrast shadow" :style="{borderColor: selectedComponent == component ? 'var(--kiss-color-primary)':''}" v-for="meta, component in filtered">
                    <kiss-row class="kiss-flex-middle" gap="small">
                        <div>
                            <kiss-svg :src="$baseUrl(meta.icon)" width="35" height="35" :style="{color: getGroupColor(meta.group)}" v-if="meta.icon"></kiss-svg>
                            <app-avatar size="35" :name="meta.label || component" :color="getGroupColor(meta.group)" v-if="!meta.icon"></app-avatar>
                        </div>
                        <div class="kiss-size-small kiss-flex-1">
                            <div class="kiss-text-bold kiss-text-truncate">{{ meta.label || component }}</div>
                            <div class="kiss-size-xsmall kiss-color-muted">{{ meta.group || '' }}</div>
                        </div>
                    </kiss-row>
                    <div class="kiss-margin-small-top kiss-flex kiss-flex-center kiss-position-relative kiss-bgcolor-transparentimage" v-if="components[component].image">
                        <display-image class="kiss-display-block kiss-margin-auto" :src="previewUrl(component)" w="600"></display-image>
                    </div>
                    <a class="kiss-cover" @click="select(component)" :title="meta.label || component"></a>
                </kiss-card>

            </div>

            <div class="kiss-padding" v-if="clipboardComponent">
                <button class="kiss-button kiss-width-1-1" @click="selectFromClipboard">{{ t('Add component from clipboard') }}</button>
            </div>

        </div>
    `
}

export default pickComponent;
